/**
 * 
 */
package com.nokia.uid;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Properties;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.log4j.Logger;

public class LogSender {

    private Queue queue;
    private QueueConnectionFactory qConnectionFactory;
    private static final Logger LOGGER = Logger.getLogger(LogSender.class);

    public LogSender(String jndiPropertiesFile) throws IOException, NamingException {
        LOGGER.info("File path: " + jndiPropertiesFile);
        FileInputStream fileInputStream = null;
        Properties properties = null;
        try {
            fileInputStream = new FileInputStream(jndiPropertiesFile);
            properties = loadProperties(fileInputStream);
        } finally {
            fileInputStream.close();
        }
        doLookUps(getInitialContext(properties), properties.getProperty("JNDI_NAME_Q"), properties.getProperty("JNDI_NAME_Q_CONNECTION_FACTORY"));
    }

    public LogSender() throws NamingException, IOException {
        Properties properties = loadProperties(this.getClass().getResourceAsStream("/client-jndi.properties"));
        doLookUps(getInitialContext(properties), properties.getProperty("JNDI_NAME_Q"), properties.getProperty("JNDI_NAME_Q_CONNECTION_FACTORY"));
    }

    public void send(String message) throws JMSException {
        LOGGER.info("Message to send: " + message);
        QueueConnection connection = null;
        QueueSession queueSession = null;
        try {
            connection = qConnectionFactory.createQueueConnection();
            queueSession = connection.createQueueSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            TextMessage textMessage = queueSession.createTextMessage();
            textMessage.setText(message);
            QueueSender queueSender = queueSession.createSender(queue);

            queueSender.send(textMessage);
            QueueBrowser browser = queueSession.createBrowser(queue);
            Enumeration enumer = browser.getEnumeration();
            int i = 0;
            while (enumer.hasMoreElements()) {
                i++;
                enumer.nextElement();
            }
            LOGGER.info("Number of messages in Q: " + i);
//            browser.close();
//            queueSender.close();

//        MessageConsumer messageConsumer = queueSession.createConsumer(queue);
//        connection.start();
//        TextMessage messageReceived = (TextMessage) messageConsumer.receive(5000);
//        LOGGER.info("Received message: " + messageReceived.getText());

        } finally {
            if(queueSession != null) {
                queueSession.close();
            }

            if(connection != null) {
                connection.close();
            }
        }

    }

    private Properties loadProperties(InputStream inputStream) throws IOException {
        Properties properties = new Properties();
        properties.load(inputStream);

        LOGGER.info("Provider URL:" + properties.getProperty("java.naming.provider.url"));
        LOGGER.info("JNDI name for Q:" + properties.getProperty("JNDI_NAME_Q"));
        LOGGER.info("JNDI name for QueueConnectionFactory:" + properties.getProperty("JNDI_NAME_Q_CONNECTION_FACTORY"));
        return properties;
    }

    private void doLookUps(Context context, String qJNDIName, String qConFcatoryJNDIName) throws NamingException {
        qConnectionFactory = (QueueConnectionFactory) context.lookup(qConFcatoryJNDIName);
        queue = (Queue) context.lookup(qJNDIName);
        context.close();
    }

    private InitialContext getInitialContext(Properties properties) throws NamingException, FileNotFoundException, IOException {
        return new InitialContext(properties);
    }
}
